@@ -27,13 +27,12 @@
 #include <glib/gi18n-lib.h>
 #include <gtk/gtk.h>
 #include <telepathy-glib/util.h>
-#include <telepathy-logger/log-manager.h>
+
 #include <folks/folks.h>
 #include <folks/folks-telepathy.h>
 
 #include <libempathy/empathy-call-factory.h>
 #include <libempathy/empathy-dispatcher.h>
-#include <libempathy/empathy-contact-manager.h>
 #include <libempathy/empathy-individual-manager.h>
 #include <libempathy/empathy-chatroom-manager.h>
 #include <libempathy/empathy-utils.h>
@@ -42,13 +41,34 @@
 #include "empathy-images.h"
 #include "empathy-log-window.h"
 #include "empathy-contact-dialogs.h"
-#include "empathy-individual-dialogs.h"
+#include "empathy-gtk-enum-types.h"
 #include "empathy-individual-edit-dialog.h"
 #include "empathy-individual-information-dialog.h"
 #include "empathy-ui-utils.h"
 #include "empathy-share-my-desktop.h"
 #include "empathy-linking-dialog.h"
 
+#define GET_PRIV(obj) EMPATHY_GET_PRIV (obj, EmpathyIndividualMenu)
+
+typedef struct {
+  FolksIndividual *individual; /* owned */
+  EmpathyIndividualFeatureFlags features;
+} EmpathyIndividualMenuPriv;
+
+enum {
+  PROP_INDIVIDUAL = 1,
+  PROP_FEATURES,
+};
+
+enum {
+  SIGNAL_LINK_CONTACTS_ACTIVATED,
+  LAST_SIGNAL
+};
+
+static guint signals[LAST_SIGNAL];
+
+G_DEFINE_TYPE (EmpathyIndividualMenu, empathy_individual_menu, GTK_TYPE_MENU);
+
 static void
 individual_menu_add_personas (GtkMenuShell *menu,
     FolksIndividual *individual,
@@ -173,29 +193,38 @@ individual_menu_add_personas (GtkMenuShell *menu,
     }
 }
 
-GtkWidget *
-empathy_individual_menu_new (FolksIndividual *individual,
-    EmpathyIndividualFeatureFlags features)
+static void
+individual_link_menu_item_activate_cb (EmpathyIndividualMenu *self)
 {
-  GtkWidget *menu;
+  EmpathyIndividualMenuPriv *priv = GET_PRIV (self);
+  GtkWidget *dialog;
+
+  dialog = empathy_linking_dialog_show (priv->individual, NULL);
+  g_signal_emit (self, signals[SIGNAL_LINK_CONTACTS_ACTIVATED], 0, dialog);
+}
+
+static void
+empathy_individual_menu_init (EmpathyIndividualMenu *self)
+{
+  EmpathyIndividualMenuPriv *priv = G_TYPE_INSTANCE_GET_PRIVATE (self,
+      EMPATHY_TYPE_INDIVIDUAL_MENU, EmpathyIndividualMenuPriv);
+
+  self->priv = priv;
+}
+
+static void
+constructed (GObject *object)
+{
+  EmpathyIndividualMenuPriv *priv = GET_PRIV (object);
   GtkMenuShell *shell;
   GtkWidget *item;
+  FolksIndividual *individual;
+  EmpathyIndividualFeatureFlags features;
 
-  g_return_val_if_fail (FOLKS_IS_INDIVIDUAL (individual), NULL);
-
-  if (features == EMPATHY_INDIVIDUAL_FEATURE_NONE)
-    return NULL;
-
-  menu = gtk_menu_new ();
-  shell = GTK_MENU_SHELL (menu);
-
-  /* Add Contact */
-  item = empathy_individual_add_menu_item_new (individual);
-  if (item)
-    {
-      gtk_menu_shell_append (shell, item);
-      gtk_widget_show (item);
-    }
+  /* Build the menu */
+  shell = GTK_MENU_SHELL (object);
+  individual = priv->individual;
+  features = priv->features;
 
   /* Chat */
   if (features & EMPATHY_INDIVIDUAL_FEATURE_CHAT)
@@ -247,7 +276,7 @@ empathy_individual_menu_new (FolksIndividual *individual,
   gtk_widget_show (item);
 
   /* Menu items to target specific contacts */
-  individual_menu_add_personas (GTK_MENU_SHELL (menu), individual, features);
+  individual_menu_add_personas (GTK_MENU_SHELL (object), individual, features);
 
   /* Separator */
   if (features & (EMPATHY_INDIVIDUAL_FEATURE_EDIT |
@@ -273,6 +302,10 @@ empathy_individual_menu_new (FolksIndividual *individual,
     {
       item = empathy_individual_link_menu_item_new (individual);
       gtk_menu_shell_append (shell, item);
+
+      g_signal_connect_swapped (item, "activate",
+          (GCallback) individual_link_menu_item_activate_cb, object);
+
       gtk_widget_show (item);
     }
 
@@ -291,113 +324,145 @@ empathy_individual_menu_new (FolksIndividual *individual,
       gtk_menu_shell_append (shell, item);
       gtk_widget_show (item);
     }
-
-  return menu;
 }
 
 static void
-empathy_individual_add_menu_item_activated (GtkMenuItem *item,
-  FolksIndividual *individual)
+get_property (GObject *object,
+    guint param_id,
+    GValue *value,
+    GParamSpec *pspec)
 {
-  GtkWidget *toplevel;
+  EmpathyIndividualMenuPriv *priv;
 
-  toplevel = gtk_widget_get_toplevel (GTK_WIDGET (item));
-  if (!gtk_widget_is_toplevel (toplevel) || !GTK_IS_WINDOW (toplevel))
-    toplevel = NULL;
+  priv = GET_PRIV (object);
 
-  empathy_new_individual_dialog_show_with_individual (GTK_WINDOW (toplevel),
-      individual);
+  switch (param_id)
+    {
+      case PROP_INDIVIDUAL:
+        g_value_set_object (value, priv->individual);
+        break;
+      case PROP_FEATURES:
+        g_value_set_flags (value, priv->features);
+        break;
+      default:
+        G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
+        break;
+    }
+}
+
+static void
+set_property (GObject *object,
+    guint param_id,
+    const GValue *value,
+    GParamSpec *pspec)
+{
+  EmpathyIndividualMenuPriv *priv;
+
+  priv = GET_PRIV (object);
+
+  switch (param_id)
+    {
+      case PROP_INDIVIDUAL:
+        priv->individual = g_value_dup_object (value);
+        break;
+      case PROP_FEATURES:
+        priv->features = g_value_get_flags (value);
+        break;
+      default:
+        G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
+        break;
+    }
+}
+
+static void
+dispose (GObject *object)
+{
+  EmpathyIndividualMenuPriv *priv = GET_PRIV (object);
+
+  tp_clear_object (&priv->individual);
+
+  G_OBJECT_CLASS (empathy_individual_menu_parent_class)->dispose (object);
+}
+
+static void
+empathy_individual_menu_class_init (EmpathyIndividualMenuClass *klass)
+{
+  GObjectClass *object_class = G_OBJECT_CLASS (klass);
+
+  object_class->constructed = constructed;
+  object_class->get_property = get_property;
+  object_class->set_property = set_property;
+  object_class->dispose = dispose;
+
+  /**
+   * EmpathyIndividualMenu:individual:
+   *
+   * The #FolksIndividual the menu is for.
+   */
+  g_object_class_install_property (object_class, PROP_INDIVIDUAL,
+      g_param_spec_object ("individual",
+          "Individual",
+          "The #FolksIndividual the menu is for.",
+          FOLKS_TYPE_INDIVIDUAL,
+          G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY | G_PARAM_STATIC_STRINGS));
+
+  /**
+   * EmpathyIndividualMenu:features:
+   *
+   * A set of feature flags controlling which entries are shown.
+   */
+  g_object_class_install_property (object_class, PROP_FEATURES,
+      g_param_spec_flags ("features",
+          "Features",
+          "A set of feature flags controlling which entries are shown.",
+          EMPATHY_TYPE_INDIVIDUAL_FEATURE_FLAGS,
+          EMPATHY_INDIVIDUAL_FEATURE_NONE,
+          G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY | G_PARAM_STATIC_STRINGS));
+
+  signals[SIGNAL_LINK_CONTACTS_ACTIVATED] =
+      g_signal_new ("link-contacts-activated", G_OBJECT_CLASS_TYPE (klass),
+          G_SIGNAL_RUN_LAST, 0, NULL, NULL,
+          g_cclosure_marshal_VOID__OBJECT,
+          G_TYPE_NONE, 1, EMPATHY_TYPE_LINKING_DIALOG);
+
+  g_type_class_add_private (object_class, sizeof (EmpathyIndividualMenuPriv));
 }
 
 GtkWidget *
-empathy_individual_add_menu_item_new (FolksIndividual *individual)
+empathy_individual_menu_new (FolksIndividual *individual,
+    EmpathyIndividualFeatureFlags features)
 {
-  GtkWidget *item;
-  GtkWidget *image;
-  EmpathyIndividualManager *manager = NULL;
-  EmpathyContact *contact = NULL;
-  TpConnection *connection;
-  GList *l, *members;
-  gboolean found = FALSE;
-  EmpathyIndividualManagerFlags flags;
-
   g_return_val_if_fail (FOLKS_IS_INDIVIDUAL (individual), NULL);
+  g_return_val_if_fail (features != EMPATHY_INDIVIDUAL_FEATURE_NONE, NULL);
 
-  if (!empathy_individual_manager_initialized ())
-    {
-      item = NULL;
-      goto out;
-    }
-
-  manager = empathy_individual_manager_dup_singleton ();
-  contact = empathy_contact_dup_from_folks_individual (individual);
-  connection = empathy_contact_get_connection (contact);
-
-  flags = empathy_individual_manager_get_flags_for_connection (manager,
-      connection);
-
-  if (!(flags & EMPATHY_INDIVIDUAL_MANAGER_CAN_ADD))
-    {
-      item = NULL;
-      goto out;
-    }
-
-  members = empathy_individual_manager_get_members (
-      EMPATHY_INDIVIDUAL_MANAGER (manager));
-
-  for (l = members; l && !found; l = l->next)
-    {
-      if (!tp_strdiff (folks_individual_get_id (l->data),
-              folks_individual_get_id (individual)))
-        {
-          found = TRUE;
-        }
-    }
-  g_list_free (members);
-
-  if (found)
-    {
-      item = NULL;
-      goto out;
-    }
-
-  item = gtk_image_menu_item_new_with_mnemonic (_("_Add Contact…"));
-  image = gtk_image_new_from_icon_name (GTK_STOCK_ADD, GTK_ICON_SIZE_MENU);
-  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), image);
-
-  g_signal_connect (item, "activate",
-      G_CALLBACK (empathy_individual_add_menu_item_activated),
-      individual);
-
-out:
-  tp_clear_object (&contact);
-  tp_clear_object (&manager);
-
-  return item;
+  return g_object_new (EMPATHY_TYPE_INDIVIDUAL_MENU,
+      "individual", individual,
+      "features", features,
+      NULL);
 }
 
-typedef gboolean (*SensitivityPredicate) (EmpathyContact *contact);
-
 /* Like menu_item_set_first_contact(), but always operates upon the given
- * contact */
+ * contact. If the contact is non-NULL, it is assumed that the menu entry should
+ * be sensitive. */
 static gboolean
 menu_item_set_contact (GtkWidget *item,
     EmpathyContact *contact,
     GCallback activate_callback,
-    SensitivityPredicate sensitivity_predicate)
+    EmpathyActionType action_type)
 {
-  gboolean contact_valid = TRUE;
+  gboolean can_do_action = FALSE;
 
-  if (sensitivity_predicate != NULL)
+  if (contact != NULL)
+    can_do_action = empathy_contact_can_do_action (contact, action_type);
+  gtk_widget_set_sensitive (item, can_do_action);
+
+  if (can_do_action == TRUE)
     {
-      contact_valid = sensitivity_predicate (contact);
-      gtk_widget_set_sensitive (item, sensitivity_predicate (contact));
+      g_signal_connect (item, "activate", G_CALLBACK (activate_callback),
+          contact);
     }
 
-  g_signal_connect (item, "activate", G_CALLBACK (activate_callback),
-      contact);
-
-  return contact_valid;
+  return can_do_action;
 }
 
 /**
@@ -405,62 +470,22 @@ menu_item_set_contact (GtkWidget *item,
  * (associated with @individual) with the highest availability who is also valid
  * whenever @item is activated.
  *
- * @sensitivity_predicate is an optional function to determine whether the menu
- * item should be insensitive (if the function returns @FALSE). Otherwise, the
- * menu item's sensitivity will not change.
+ * @action_type is the type of action performed by the menu entry; this is used
+ * so that only contacts which can perform that action (e.g. are capable of
+ * receiving video calls) are selected, as appropriate.
  */
 static GtkWidget *
 menu_item_set_first_contact (GtkWidget *item,
     FolksIndividual *individual,
     GCallback activate_callback,
-    SensitivityPredicate sensitivity_predicate)
+    EmpathyActionType action_type)
 {
-  GList *personas, *l;
-  FolksPresenceType best_presence = FOLKS_PRESENCE_TYPE_UNSET;
-  EmpathyContact *best_contact = NULL;
+  EmpathyContact *best_contact;
 
-  personas = folks_individual_get_personas (individual);
-  for (l = personas; l != NULL; l = l->next)
-    {
-      TpfPersona *persona = l->data;
-      FolksPresenceType presence;
-
-      if (!TPF_IS_PERSONA (persona))
-        continue;
-
-      /* Only choose the contact if it has a higher presence than our current
-       * best choice of contact. */
-      presence = folks_presence_get_presence_type (FOLKS_PRESENCE (l->data));
-      if (folks_presence_typecmp (presence, best_presence) > 0)
-        {
-          TpContact *tp_contact;
-          EmpathyContact *contact;
-
-          tp_contact = tpf_persona_get_contact (TPF_PERSONA (l->data));
-          contact = empathy_contact_dup_from_tp_contact (tp_contact);
-          empathy_contact_set_persona (contact, FOLKS_PERSONA (l->data));
-
-          if (best_contact == NULL || sensitivity_predicate == NULL ||
-              sensitivity_predicate (contact) == TRUE)
-            {
-              tp_clear_object (&best_contact);
-
-              best_presence = presence;
-              best_contact = g_object_ref (contact);
-            }
-
-          g_object_unref (contact);
-        }
-    }
-
-  /* Use the best contact we found */
-  if (best_contact != NULL)
-    {
-      menu_item_set_contact (item, best_contact, G_CALLBACK (activate_callback),
-          sensitivity_predicate);
-
-      g_object_unref (best_contact);
-    }
+  best_contact = empathy_contact_dup_best_for_action (individual, action_type);
+  menu_item_set_contact (item, best_contact, G_CALLBACK (activate_callback),
+      action_type);
+  tp_clear_object (&best_contact);
 
   return item;
 }
@@ -495,12 +520,14 @@ empathy_individual_chat_menu_item_new (FolksIndividual *individual,
   if (contact != NULL)
     {
       menu_item_set_contact (item, contact,
-          G_CALLBACK (empathy_individual_chat_menu_item_activated), NULL);
+          G_CALLBACK (empathy_individual_chat_menu_item_activated),
+          EMPATHY_ACTION_CHAT);
     }
   else
     {
       menu_item_set_first_contact (item, individual,
-          G_CALLBACK (empathy_individual_chat_menu_item_activated), NULL);
+          G_CALLBACK (empathy_individual_chat_menu_item_activated),
+          EMPATHY_ACTION_CHAT);
     }
 
   return item;
@@ -536,13 +563,13 @@ empathy_individual_audio_call_menu_item_new (FolksIndividual *individual,
     {
       menu_item_set_contact (item, contact,
           G_CALLBACK (empathy_individual_audio_call_menu_item_activated),
-          empathy_contact_can_voip_audio);
+          EMPATHY_ACTION_AUDIO_CALL);
     }
   else
     {
       menu_item_set_first_contact (item, individual,
           G_CALLBACK (empathy_individual_audio_call_menu_item_activated),
-          empathy_contact_can_voip_audio);
+          EMPATHY_ACTION_AUDIO_CALL);
     }
 
   return item;
@@ -579,13 +606,13 @@ empathy_individual_video_call_menu_item_new (FolksIndividual *individual,
     {
       menu_item_set_contact (item, contact,
           G_CALLBACK (empathy_individual_video_call_menu_item_activated),
-          empathy_contact_can_voip_video);
+          EMPATHY_ACTION_VIDEO_CALL);
     }
   else
     {
       menu_item_set_first_contact (item, individual,
           G_CALLBACK (empathy_individual_video_call_menu_item_activated),
-          empathy_contact_can_voip_video);
+          EMPATHY_ACTION_VIDEO_CALL);
     }
 
   return item;
@@ -601,21 +628,6 @@ empathy_individual_log_menu_item_activated (GtkMenuItem *item,
       empathy_contact_get_id (contact), FALSE, NULL);
 }
 
-static gboolean
-contact_has_log (EmpathyContact *contact)
-{
-  TplLogManager *manager;
-  gboolean have_log;
-
-  manager = tpl_log_manager_dup_singleton ();
-  have_log = tpl_log_manager_exists (manager,
-      empathy_contact_get_account (contact), empathy_contact_get_id (contact),
-      FALSE);
-  g_object_unref (manager);
-
-  return have_log;
-}
-
 GtkWidget *
 empathy_individual_log_menu_item_new (FolksIndividual *individual,
     EmpathyContact *contact)
@@ -636,13 +648,13 @@ empathy_individual_log_menu_item_new (FolksIndividual *individual,
     {
       menu_item_set_contact (item, contact,
           G_CALLBACK (empathy_individual_log_menu_item_activated),
-          contact_has_log);
+          EMPATHY_ACTION_VIEW_LOGS);
     }
   else
     {
       menu_item_set_first_contact (item, individual,
           G_CALLBACK (empathy_individual_log_menu_item_activated),
-          contact_has_log);
+          EMPATHY_ACTION_VIEW_LOGS);
     }
 
   return item;
@@ -678,13 +690,13 @@ empathy_individual_file_transfer_menu_item_new (FolksIndividual *individual,
     {
       menu_item_set_contact (item, contact,
           G_CALLBACK (empathy_individual_file_transfer_menu_item_activated),
-          empathy_contact_can_send_files);
+          EMPATHY_ACTION_SEND_FILE);
     }
   else
     {
       menu_item_set_first_contact (item, individual,
           G_CALLBACK (empathy_individual_file_transfer_menu_item_activated),
-          empathy_contact_can_send_files);
+          EMPATHY_ACTION_SEND_FILE);
     }
 
   return item;
@@ -719,13 +731,13 @@ empathy_individual_share_my_desktop_menu_item_new (FolksIndividual *individual,
     {
       menu_item_set_contact (item, contact,
           G_CALLBACK (empathy_individual_share_my_desktop_menu_item_activated),
-          empathy_contact_can_use_rfb_stream_tube);
+          EMPATHY_ACTION_SHARE_MY_DESKTOP);
     }
   else
     {
       menu_item_set_first_contact (item, individual,
           G_CALLBACK (empathy_individual_share_my_desktop_menu_item_activated),
-          empathy_contact_can_use_rfb_stream_tube);
+          EMPATHY_ACTION_SHARE_MY_DESKTOP);
     }
 
   return item;
@@ -837,12 +849,6 @@ empathy_individual_edit_menu_item_new (FolksIndividual *individual)
   return item;
 }
 
-static void
-individual_link_menu_item_activate_cb (FolksIndividual *individual)
-{
-  empathy_linking_dialog_show (individual, NULL);
-}
-
 GtkWidget *
 empathy_individual_link_menu_item_new (FolksIndividual *individual)
 {
@@ -854,7 +860,7 @@ empathy_individual_link_menu_item_new (FolksIndividual *individual)
   item = gtk_image_menu_item_new_with_mnemonic (
       /* Translators: this is a verb meaning "to connect two contacts together
        * to form a meta-contact". */
-      C_("Link individual (contextual menu)", "_Link…"));
+      C_("Link individual (contextual menu)", "_Link Contacts…"));
   /* TODO */
   /*image = gtk_image_new_from_icon_name (GTK_STOCK_EDIT, GTK_ICON_SIZE_MENU);
   gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), image);
@@ -865,9 +871,6 @@ empathy_individual_link_menu_item_new (FolksIndividual *individual)
       folks_individual_get_trust_level (individual) ==
           FOLKS_TRUST_LEVEL_PERSONAS);
 
-  g_signal_connect_swapped (item, "activate",
-      G_CALLBACK (individual_link_menu_item_activate_cb), individual);
-
   return item;
 }
 
