@@ -201,8 +201,8 @@ empathy_account_settings_constructed (GObject *object)
     {
       tp_account_prepare_async (priv->account, NULL,
           empathy_account_settings_account_ready_cb, self);
-      empathy_signal_connect_weak (priv->managers, "notify::ready",
-        G_CALLBACK (empathy_account_settings_managers_ready_cb), object);
+      tp_g_signal_connect_object (priv->managers, "notify::ready",
+        G_CALLBACK (empathy_account_settings_managers_ready_cb), object, 0);
     }
 
   if (G_OBJECT_CLASS (
@@ -682,6 +682,20 @@ empathy_account_settings_get_string (EmpathyAccountSettings *settings,
   return g_value_get_string (v);
 }
 
+const gchar * const *
+empathy_account_settings_get_strv (EmpathyAccountSettings *settings,
+    const gchar *param)
+{
+  const GValue *v;
+
+  v = empathy_account_settings_get (settings, param);
+
+  if (v == NULL || !G_VALUE_HOLDS (v, G_TYPE_STRV))
+    return NULL;
+
+  return g_value_get_boxed (v);
+}
+
 gint32
 empathy_account_settings_get_int32 (EmpathyAccountSettings *settings,
     const gchar *param)
@@ -856,6 +870,18 @@ empathy_account_settings_set_string (EmpathyAccountSettings *settings,
   account_settings_remove_from_unset (settings, param);
 }
 
+void
+empathy_account_settings_set_strv (EmpathyAccountSettings *settings,
+    const gchar *param,
+    gchar **value)
+{
+  EmpathyAccountSettingsPriv *priv = GET_PRIV (settings);
+
+  tp_asv_set_strv (priv->parameters, param, value);
+
+  account_settings_remove_from_unset (settings, param);
+}
+
 void
 empathy_account_settings_set_int32 (EmpathyAccountSettings *settings,
     const gchar *param,
@@ -950,6 +976,13 @@ empathy_account_settings_set_display_name_async (
   result = g_simple_async_result_new (G_OBJECT (settings),
       callback, user_data, empathy_account_settings_set_display_name_finish);
 
+  if (!tp_strdiff (name, priv->display_name))
+    {
+      /* Nothing to do */
+      g_simple_async_result_complete_in_idle (result);
+      return;
+    }
+
   if (priv->account == NULL)
     {
       if (priv->display_name != NULL)
@@ -1237,13 +1270,21 @@ empathy_account_settings_has_account (EmpathyAccountSettings *settings,
     TpAccount *account)
 {
   EmpathyAccountSettingsPriv *priv;
+  const gchar *account_path;
+  const gchar *priv_account_path;
 
   g_return_val_if_fail (EMPATHY_IS_ACCOUNT_SETTINGS (settings), FALSE);
   g_return_val_if_fail (TP_IS_ACCOUNT (account), FALSE);
 
   priv = GET_PRIV (settings);
 
-  return (account == priv->account);
+  if (priv->account == NULL)
+    return FALSE;
+
+  account_path = tp_proxy_get_object_path (TP_PROXY (account));
+  priv_account_path = tp_proxy_get_object_path (TP_PROXY (priv->account));
+
+  return (!tp_strdiff (account_path, priv_account_path));
 }
 
 gboolean
